# -*- coding: utf-8 -*-
#    Copyright (C) 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Python interface to 'crontab' binary
"""

import os
from fwbackups.utils import execute, executeSub
from fwbackups.i18n import _

class CronError(Exception):
    """ Cron interfacing error. """
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class CronLine:
    """ Implement later, add stuff like validation """
    def __init__(self):
        pass

class CronTab:
    """ The class for in iterfacing with the crontab """
    def __init__(self):
        """ Initialize it all """
        # Set me before using me!
        self.environ = None
        self.read()

    def read(self):
        """ Read the crontab """
        sub = executeSub(['crontab', '-l'], self.environ)
        retval = sub.wait(3)
        if retval != os.EX_OK and retval != 1:
            raise CronError('stderr:\n%sstdout:\n%s' % (sub.stderr.readlines(), sub.stdout.readlines()))
        return sub.stdout.readlines()

    def clean(self):
        """ Removes a user's crontab """
        a, b, c = execute(['crontab -l | grep -v "fwbackups-run" > /tmp/cron-nonfwbackups.txt'], self.environ, shell=True)
        sub = executeSub(['crontab', '-r'], self.environ)
        fh = open('/tmp/cron-nonfwbackups.txt', 'r')
        self.write(''.join(fh.readlines()))
        fh.close()
        os.remove('/tmp/cron-nonfwbackups.txt')
        return sub.wait(3)

    def write(self, content):
        """ Write the crontab """
        sub = executeSub(['crontab', '-e'], self.environ)
        sub.stdin.write(content)
        sub.stdin.close()
        retval = sub.wait(3)
        if retval != os.EX_OK:
            stdout = ' '.join(sub.stdout.readlines())
            stderr = ' '.join(sub.stderr.readlines())
            raise CronError(_('Could not write \'%(a)s\' to crontab: %(b)s%(c)s') % {'a': content, 'b': stdout, 'c': stderr})

    def search(self, line):
        """ Search for an entry """
        line = ' '.join(line)
        contents = self.read()
        for i in contents:
            if i.strip() == line:
                return True
        return False

    def add(self, line):
        """ Add a crontab entry """
        contents = ''.join(self.read())
        contents += ' '.join(line)
        self.write(contents)

    def edit(self, oldline, newline):
        """ Edit a crontab entry """
        if not self.remove(oldline):
            raise CronError(_('No matches found for \'%s\'.') % oldline)
        self.add(newline)

    def remove(self, line):
        """ Remove a crontab entry """
        line = ' '.join(line)
        contents = self.read()
        for i in contents:
            if i.strip() == line:
                try:
                    contents.remove(i)
                except:
                    return False
                self.write(''.join(contents))
                return True
        return False

