# -*- coding: utf-8 -*-
#    Copyright (C) 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Parts (C) Thomas Leonard (taken from ROX-lib2)
"""
Classes used by the GUI, mostly to make callbacks easier
"""
import gtk, gtk.glade, gobject
import logging
import pango
import types
import time
import re
import codecs
import webbrowser

from i18n import _
from const import *
import cron, utils, guiutils, config, backend

class TextViewConsole:
    """ Encapsulate a gtk.TextView """
    def __init__(self, textview, default_style=None, font=None, color=None):
        """ Initialize, define styles """
        self.textview = textview
        self.buffer = self.textview.get_buffer()
        self.endMark = self.buffer.create_mark("End", self.buffer.get_end_iter(), False)
        self.startMark = self.buffer.create_mark("Start", self.buffer.get_start_iter(), False)
        #setup styles.
        self.style_banner = gtk.TextTag("banner")
        self.style_banner.set_property("foreground", "saddle brown")
        self.style_banner.set_property("family", "Monospace")
        self.style_banner.set_property("size_points", 8)
        
            
        self.style_ps1 = gtk.TextTag("ps1")
        self.style_ps1.set_property("editable", False)
        if color:
            self.style_ps1.set_property("foreground", color)
        else:
            self.style_ps1.set_property("foreground", "DarkOrchid4")
        if font:
            self.style_ps1.set_property("font", font)
        else:
            self.style_ps1.set_property("family", "Monospace")
            self.style_ps1.set_property("size_points", 8)

        self.style_ps2 = gtk.TextTag("ps2")
        self.style_ps2.set_property("foreground", "DarkOliveGreen")
        self.style_ps2.set_property("editable", False)
        self.style_ps2.set_property("font", "Monospace")

        self.style_out = gtk.TextTag("stdout")
        self.style_out.set_property("foreground", "midnight blue")
        self.style_out.set_property("family", "Monospace")
        self.style_out.set_property("size_points", 8)


        self.style_err = gtk.TextTag("stderr") 
        #self.style_err.set_property("style", pango.STYLE_ITALIC)
        self.style_err.set_property("foreground", "red")
        if font:
            self.style_err.set_property("font", font)
        else:
            self.style_err.set_property("family", "Monospace")
            self.style_err.set_property("size_points", 8)

        self.style_warn = gtk.TextTag("warn")
        self.style_warn.set_property("foreground", "darkorange")
        if font:
            self.style_warn.set_property("font", font)
        else:
            self.style_warn.set_property("family", "Monospace")
            self.style_warn.set_property("size_points", 8)

        self.buffer.get_tag_table().add(self.style_banner)
        self.buffer.get_tag_table().add(self.style_ps1)
        self.buffer.get_tag_table().add(self.style_ps2)
        self.buffer.get_tag_table().add(self.style_out)
        self.buffer.get_tag_table().add(self.style_err)
        self.buffer.get_tag_table().add(self.style_warn)
        
        if default_style:
            self.default_style=default_style
        else:
            self.default_style=self.style_out
    
    def changeStyle(self,color,font,style=None):
        """ Change the text style """
        if not style:
            self.default_style.set_property("foreground", color)
            self.default_style.set_property("font", font)
        else:
            style.set_property("foreground", color)
            style.set_property("font", font)
    
    def write_line(self, txt, style=None):
        """ Write a line to the bottom of textview and scoll to end.
            txt: Text to write
            style: (optional) Predefinded pango style to use.
        """
        #txt = gobject.markup_escape_text(txt)
        txt = self._toUTF(txt)
        start, end = self.buffer.get_bounds()
        if style == None:
            self.buffer.insert_with_tags(end, txt, self.default_style)
        else:
            self.buffer.insert_with_tags(end, txt, style)
        self.goBottom()

    def _toUTF(self, txt):
        rc=""
        try:
            rc = unicode(txt, 'utf-8')
        except UnicodeDecodeError, e:
            rc = unicode(txt, 'iso-8859-1')
        return rc

    def clear(self):
        self.buffer.set_text('')
        
    def goTop(self):
        self.textview.scroll_to_iter(self.buffer.get_start_iter(), 0.0)

    def goBottom(self):
        self.textview.scroll_to_iter(self.buffer.get_end_iter(), 0.0)

class TextViewLogHandler(logging.FileHandler):
    """ Python logging handler for writing in a TextViewConsole AND a file """
    def __init__(self, console):
        logging.FileHandler.__init__(self, logloc)
        self.console = console
        self.history()
        self.stream = None
        self.console.write_line('-- %s --%s%s' % (_('Current session\'s log \
messages'), linesep, linesep), self.console.style_ps2)

    def history(self):
        """ Get + print all old log messages stored in the log file """
        stream = open(logloc, 'r')
        text = stream.read()
        stream.close()
        if text.strip() != '':
            self.console.write_line('-- %s --%s%s' % (_('Previous log messages')\
, linesep, linesep), self.console.style_ps2)
            for i in text.split(linesep):
                errType = i.split(' ')
                if len(errType) >= 5:
                    if errType[4] == _('CRITICAL') or errType[4] == _('ERROR'):
                        self.console.write_line('%s%s' % (i, linesep), self.console.style_err)
                    elif errType[4] == _('WARNING'):
                        self.console.write_line('%s%s' % (i, linesep), self.console.style_warn)
                    else:
                        self.console.write_line('%s%s' % (i, linesep), self.console.style_ps1)
                else:
                    self.console.write_line('%s%s' % (i, linesep), self.console.style_ps1)
        else:
            self.console.write_line('-- %s --%s' % (_('No previous log messages to display'),\
linesep), self.console.style_ps2)
        
    def emit(self, record):
        """ Emit a message """
        self.stream = open(logloc, 'a')
        msg = self.format(record)
        
        if self.console:
            if record.levelno < 30:
                self.console.write_line("%s\n" % msg)
            elif record.levelno == 30:
                self.console.write_line("%s\n" % msg,self.console.style_warn)
            else:
                self.console.write_line("%s\n" % msg,self.console.style_err)
        fs = "%s\n"
        if not hasattr(types, "UnicodeType"): #if no unicode support...
            self.stream.write(fs % msg)
        else:
            try:
                self.stream.write(fs % msg)
            except UnicodeError:
                self.stream.write(fs % msg.encode("UTF-8"))
        self.flush()

    def clear(self):
        """ Clear the log file and the text view """
        self.console.clear()
        stream = open(logloc, 'w')
        stream.write('')
        stream.close()
        if self.stream:
            self.stream.flush()
        self.history()


###################################################


class StatusBar:
    def __init__(self, widget):
        """ Initialize.
            widget: GTK widget to use as the statusbar
        """
        self.statusbar = widget

    def newmessage(self, message, seconds=3):
        """ New message in status bar.
            message: Message to display
            seconds: number of seconds to display the message
                     defaults to three.
        """
        try:
            self.message_timeout()
        # Just means there was no existing message.
        except AttributeError:
            pass
        self.statusbar.push(1, message)
        self.message_timer = gobject.timeout_add(seconds*1000, self.message_timeout)

    def message_timeout(self):
        """ Remove a message from the statusbar """
        self.statusbar.pop(1)
        gobject.source_remove(self.message_timer)
        return True

class ProgressBar:
    def __init__(self, widget, ms=15):
        """ Initialize.
            widget: GTK widget to use as the progressbar
            ms: Pulse step is every this many miliseconds
        """
        self.progressbar = widget
        self.ms = ms
        self.pulsing = False
        self.progressbar.set_pulse_step(0.01)

    def _pulse(self):
        """ Pulse the progressbar. """
        self.progressbar.pulse()
        # Has to return true to happen again
        return True

    def startPulse(self):
        """ Start auto-pulsing the progressbar """
        if self.pulsing:
            self.stopPulse()
        self.pulsing = True
        self.pulsetimer = gobject.timeout_add(self.ms, self._pulse)

    def stopPulse(self):
        """ Stop auto-pulsing the progressbar """
        gobject.source_remove(self.pulsetimer)
        self.progressbar.set_fraction(0)
        self.pulsing = False
        return True

    def setMs(self, ms):
        """ Set the ms value (see __init__) """ 
        self.ms = int(ms)
        if self.pulsing:
            self.stopPulse()
            self.startPulse()

    def set_text(self, text):
        """ Set the progressbar's text """
        self.progressbar.set_text(text)

###################################################


class GenericDia:
    """ Wrapper for the generic dialog """
    def __init__(self, widget):
        """ Initialize """
        self.dialog = widget
    def run(self):
        """ Run the dialog """
        self.dialog.show()
        try:
            return self.dialog.run()
        except:
            self.destroy()
            raise
    def destroy(self):
        """ Hide the dialog, don't kill it! """
        self.dialog.hide()

class PathBrowser(GenericDia):
    """ Wrapper for generic path dialogs """
    def __init__(self, widget, ffilter=None):
        """ Initialize.
            widget: The widget to use
            ffilter: List for filter creation.
                    > 1st value is filter pattern
                    > 2nd value is the description
        """
        GenericDia.__init__(self, widget)
        if ffilter:
            self.ffilter = gtk.FileFilter()
            self.ffilter.add_pattern(ffilter[0])
            self.ffilter.set_name(ffilter[1])
            self.dialog.add_filter(self.ffilter)
        else:
            self.ffilter = None

    def destroy(self):
        """ Destroy the dialog and the filter """
        GenericDia.destroy(self)
        if self.ffilter:
            self.dialog.remove_filter(self.ffilter)
    def set_current_folder(self, path):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_current_folder(path)
    def set_select_multiple(self, bool):
        """ Wrapper: See GTK+ help. """
        if bool == True:
            return self.dialog.set_select_multiple(True)
        elif bool == False:
            return self.dialog.set_select_multiple(False)
    def set_action(self, action):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_action(action)
    def set_title(self, title):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_title(title)
    def get_filename(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def get_filenames(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def set_filename(self, filename):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_filename(filename)
    def add_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.add_filter(ffilter)
    def remove_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.remove_filter(ffilter)        

class PathDia(PathBrowser):
    """ Wrapper for path selection dialog """
    def __init__(self, widget, mode, ffilter=None, multiple=True):
        """ Inititalize. 
            mode: The gtk filebowser mode (open, save, file, etc)
            ffilter: File filter to use (see PathBrowser)
            multiple: Select multiple files or folders at once? Defaults to True
        """
        PathBrowser.__init__(self, widget, ffilter)
        widget.set_action(mode)
        self.dialog.set_select_multiple(multiple)

class SaveDia(PathBrowser):
    """ Wrapper for save dialog """
    def __init__(self, widget, ffilter=None):
        """ Inititalize. """
        PathBrowser.__init__(self, widget, ffilter)
        widget.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
        self.dialog.set_select_multiple(False)

class BackupSetDia(GenericDia):
    """ Wrapper for backup set properties dialog """
    def __init__(self, widget):
        """ Initialize """
        GenericDia.__init__(self, widget)

class ConfirmDia(GenericDia):
    """ Wrapper for confirmation dialog """
    def __init__(self, widget, label, message=None, dontShowMe=False):
        """ Initialize """
        GenericDia.__init__(self, widget)
        if message:
            label.set_text(message)
            label.set_use_markup(True)
        else:
            label.set_text(_('Are you sure you would like to continue?'))
            label.set_use_markup(True)
        self.dontShowMe = dontShowMe
        if self.dontShowMe:
            self.dontShowMe.set_active(False)
            self.dontShowMe.show()

    def destroy(self):
        """ Destroy the dialog, hide checkmark """
        if self.dontShowMe:
            self.dontShowMe.hide()
        GenericDia.destroy(self)

class ErrorDia(GenericDia):
    """ Wrapper for error dialog """
    def __init__(self, widget, label, message=None):
        """ Initialize """
        GenericDia.__init__(self, widget)
        if message:
            label.set_text(message)
            label.set_use_markup(True)
        else:
            label.set_text(_('An unspecified error occurred.'))
            label.set_use_markup(True)

class InfoDia(GenericDia):
    """ Wrapper for error dialog """
    def __init__(self, widget, label, message, dontShowMe=False):
        """ Initialize """
        GenericDia.__init__(self, widget)
        label.set_text(message)
        label.set_use_markup(True)
        self.dontShowMe = dontShowMe
        if self.dontShowMe:
            self.dontShowMe.set_active(False)
            self.dontShowMe.show()

    def destroy(self):
        """ Destroy the dialog, hide checkmark """
        if self.dontShowMe:
            self.dontShowMe.hide()
        GenericDia.destroy(self)


class ExportDia(GenericDia):
    """ Wrapper for export dialog """
    def __init__(self, widget):
        """ Initialize """
        GenericDia.__init__(self, widget)

class OptionsDia(GenericDia):
    """ Wrapper for one-time options dialog """
    def __init__(self, widget):
        """ Initialize """
        GenericDia.__init__(self, widget)

class PreferenceDia(GenericDia):
    """ Wrapper for backup set properties dialog """
    def __init__(self, widget):
        """ Initialize """
        GenericDia.__init__(self, widget)


###################################################

class View:
    """ Generic view class. Should be subclassed."""
    def __init__(self):
        """ Nothing here """
        pass

    def _buildListstoreIndex(self, liststore, col):
        """ Build a list of all values in a liststore """
        items = []
        item = liststore.get_iter_first()
        while (item != None):
            items.append (liststore.get_value(item, col))
            item = liststore.iter_next(item)
        return items

    #def _checkDestPerms(self, path, imgwidget, labelwidget):
    def _checkDestPerms(self, path, imgwidget):
        """ Check perms on a path and set image + label accordingly """
        if utils.CheckPerms(path):
            imgwidget.set_from_stock(gtk.STOCK_OK, gtk.ICON_SIZE_BUTTON)
            #labelwidget.set_text(_('You have read and write permissions'))
        else:
            imgwidget.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            #labelwidget.set_text(_('You do NOT have read and write permissions'))

class PathView(View):
    """ Wrapper for paths in a treeview """
    def __init__(self, treeview, statusbar, ui):
        self.treeview = treeview
        self.statusbar = statusbar
        self.ui = ui
        self.logger = logging.getLogger(LOGGERNAME)

        # Give it columns
        cell = gtk.CellRendererPixbuf()
        col = gtk.TreeViewColumn(_('Access'), cell, stock_id=0)
        col.set_resizable(True)
        self.treeview.append_column(col)

        cell = gtk.CellRendererText()
        cell.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)
        col = gtk.TreeViewColumn(_('Path'), cell, text=1)
        col.set_resizable(True)
        self.treeview.append_column(col)

        self.liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.treeview.set_model(self.liststore)
        self.treeview.set_reorderable(False)

        # Allow enable drag and drop of rows including row move
        #self.TARGETS = [('text/plain', 0, 1)]
        #self.treeview.enable_model_drag_dest(self.TARGETS, gtk.gdk.ACTION_DEFAULT)
        target = [('text/uri-list', 0, 0)]
        self.treeview.drag_dest_set(gtk.DEST_DEFAULT_ALL, target, gtk.gdk.ACTION_COPY)

        def escape(uri):
            "Convert each space to %20, etc"
            _to_utf8 = codecs.getencoder('utf-8')
            return re.sub('[^:-_./a-zA-Z0-9]',
                lambda match: '%%%02x' % ord(match.group(0)),
                _to_utf8(uri)[0])

        def unescape(uri):
            "Convert each %20 to a space, etc"
            if '%' not in uri: return uri
            return re.sub('%[0-9a-fA-F][0-9a-fA-F]',
                lambda match: chr(int(match.group(0)[1:], 16)),
                uri)

        def get_local_path(uri):
            """Convert 'uri' to a local path and return, if possible. If 'uri'
            is a resource on a remote machine, return None. URI is in the escaped form
            (%20 for space)."""
            if not uri:
                return None

            if uri[0] == '/':
                if uri[1:2] != '/':
                    return unescape(uri)    # A normal Unix pathname
                i = uri.find('/', 2)
                if i == -1:
                    return None    # //something
                if i == 2:
                    return unescape(uri[2:])    # ///path
                remote_host = uri[2:i]
                if remote_host == our_host_name():
                    return unescape(uri[i:])    # //localhost/path
                # //otherhost/path
            elif uri[:5].lower() == 'file:':
                if uri[5:6] == '/':
                    return get_local_path(uri[5:])
            elif uri[:2] == './' or uri[:3] == '../':
                return unescape(uri)
            return None

        def drag_data_received(w, context, x, y, data, info, time):
            paths = []
            if data and data.format == 8:
                 for i in data.data.split('\r\n'):
                     if i != "" or i != None:
                         path = get_local_path(i)
                         if path != None:
                             paths.append(path)
            for i in paths:
                self.add([i], self._buildListstoreIndex(self.liststore, 1))
            context.finish(True, False, time)

        self.treeview.connect('drag_data_received', drag_data_received)
        """self.treeview.connect("drag_data_get", self.drag_data_get_data)

        def drag_data_get_data(self, treeview, context, selection, target_id, etime):
            treeselection = treeview.get_selection()
            model, iter = treeselection.get_selected()
            data = model.get_value(iter, 0)
            selection.set(selection.target, 8, data)

        def drag_data_received_data(self, treeview, context, x, y, selection, info, etime):
            model = treeview.get_model()
            data = selection.data
            print selection.type
            drop_info = treeview.get_dest_row_at_pos(x, y)
            if drop_info:
                path, position = drop_info
                iter = model.get_iter(path)
                if (position == gtk.TREE_VIEW_DROP_BEFORE
                    or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
                    model.insert_before(iter, [data])
                else:
                    model.insert_after(iter, [data])
            else:
                model.append([data])
            if context.action == gtk.gdk.ACTION_MOVE:
                context.finish(True, True, etime)
            return"""
        
        # Just to keep things clean.
        self.clear()
    
    def addFile(self):
        """ Add a file to the pathview """
        dialog = PathDia(self.ui.path_dia, gtk.FILE_CHOOSER_ACTION_OPEN, multiple=True)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.add(dialog.get_filenames(), self._buildListstoreIndex(self.liststore, 1))
        dialog.destroy()

    def addFolder(self):
        """ Add a folder to the pathview """
        dialog = PathDia(self.ui.path_dia, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, multiple=True)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.add(dialog.get_filenames(), self._buildListstoreIndex(self.liststore, 1))
        dialog.destroy()

    def add(self, paths, values):
        """ Add a generic path """
        for i in paths:
            # Only if it doesn't already exist.
            try:
                values.index(i)
            except ValueError:
                # index() fails with ValueError when not found
                if utils.CheckPermsRead(i):
                    self.liststore.append([gtk.STOCK_OK, i])
                else:
                    self.liststore.append([gtk.STOCK_CANCEL, i])

    def editPath(self):
        dialog = PathDia(\
                         self.ui.path_dia, \
                         gtk.FILE_CHOOSER_ACTION_OPEN, \
                         multiple=False\
                         )
        try:
            (model, iterator) = self.treeview.get_selection().get_selected()
            name = self.liststore.get_value(iterator, 1)
        except TypeError:
            self.statusbar.newmessage(_('Please select a path before choosing an action.'), 3)
            return
        if os.path.isdir(name):
            dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            dialog.set_current_folder(name)
        elif os.path.isfile(name):
            dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
            dialog.set_filename(name)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            name = dialog.get_filenames()[0]
            if utils.CheckPermsRead(name):
                self.liststore.set_value(iterator, 0, gtk.STOCK_OK)
                self.liststore.set_value(iterator, 1, name)
            else:
                self.liststore.set_value(iterator, 0, gtk.STOCK_CANCEL)
                self.liststore.set_value(iterator, 1, name)
        dialog.destroy()

    def removePath(self):
        try:
            (model, iterator) = self.treeview.get_selection().get_selected()
            model.remove(iterator)
        except TypeError:
            self.statusbar.newmessage(_('Please select a path before choosing an action.'), 3)
            return
    
    def clear(self):
        self.liststore.clear()

    def load(self, config):
        config.read()
        o = config.options('Paths')
        o.sort()
        for i in o:
            path = config.get('Paths', i)
            if utils.CheckPermsRead(path):
                self.liststore.append([gtk.STOCK_OK, path])
            else:
                self.liststore.append([gtk.STOCK_CANCEL, path])

    def refresh(self, config):
        self.clear()
        self.load(config)



class OneTimeView(PathView):
    """ Wrapper for the One-Time Backup view """
    def __init__(self, treeview, statusbar, ui):
        PathView.__init__(self, treeview, statusbar, ui)
        self.treeview = treeview
        self.statusbar = statusbar
        self.ui = ui
        self.logger = logging.getLogger(LOGGERNAME)
        self.ui.OneTimeDestination.set_current_folder(userhome)
        self.progressbar = ProgressBar(self.ui.OneTimeProgress)
        self._checkDestPerms(userhome, self.ui.OneTimeDestinationPermissionImage)
        if os.path.exists(onetimeloc):
            os.remove(onetimeloc)
        onetime = config.OneTimeConf(True)

    def _toggleLocked(self, ignore):
        """ Toggle locking in the UI """
        for widget in   [\
self.ui.BackupSetsRadioTool,\
self.ui.OneTimeRadioTool, self.ui.RestoreRadioTool,\
self.ui.OneTimeStartButton, self.ui.OneTimeVButtonBox,\
self.ui.OneTimeCancelButton, self.ui.OneTimeDestination,\
self.ui.OneTimeOptionsButton,\
self.ui.one_time_backup1,\
]:
            try:
                ignore.index(widget)
            except ValueError:
                widget.set_sensitive(not widget.get_property('sensitive'))
        for widget in [\
self.ui.new_set1, self.ui.backup_set_now1,\
self.ui.import_sets1,\
self.ui.export_sets1, self.ui.restore_set1,\
self.ui.restore_set2, self.ui.edit_set1,\
self.ui.remove_set1, self.ui.duplicate_set1\
]:
            widget.set_sensitive(False)


    def _saveRow(self, model, path, iter, config, user_data=False):
        """ Saves one row to the configuration """
        pathno = 'path' + str(self.pathnumber)
        if not user_data:
            user_data = model.get_value(iter, 1)
        config.set('Paths', pathno, user_data)
        self.pathnumber = int(self.pathnumber + 1)

    def _save(self):
        onetime = config.OneTimeConf()
        self.pathnumber = 0
        self.liststore.foreach(self._saveRow, onetime)
        if self.ui.OneTimeRecursiveCheck.get_active():
            onetime.set('Options', 'Recursive', 1)
        else:
            onetime.set('Options', 'Recursive', 0)
        if self.ui.OneTimeBackupHiddenCheck.get_active():
            onetime.set('Options', 'BackupHidden', 1)
        else:
            onetime.set('Options', 'BackupHidden', 0)
        if self.ui.OneTimePkgListsToFileCheck.get_active():
            onetime.set('Options', 'PkgListsToFile', 1)
        else:
            onetime.set('Options', 'PkgListsToFile', 0)
        if self.ui.OneTimeDiskInfoToFileCheck.get_active():
            onetime.set('Options', 'DiskInfoToFile', 1)
        else:
            onetime.set('Options', 'DiskInfoToFile', 0)
        if self.ui.OneTimeSparseCheck.get_active():
            onetime.set('Options', 'Sparse', 1)
        else:
            onetime.set('Options', 'Sparse', 0)
        if self.ui.OneTimeFollowLinksCheck.get_active():
            onetime.set('Options', 'FollowLinks', 1)
        else:
            onetime.set('Options', 'FollowLinks', 0)
        if self.ui.OneTimeArchiveEngineRadio.get_active():
            onetime.set('Options', 'Engine', 'tar')
        elif self.ui.OneTimeCompressedArchiveEngineRadio.get_active():
            onetime.set('Options', 'Engine', 'tar.gz')
        elif self.ui.OneTimeDirectCopyEngineRadio.get_active():
            onetime.set('Options', 'Engine', 'rsync')
        nice = int(self.ui.OneTimeNiceScale.get_value())
        onetime.set('Options', 'Nice', nice)
        # Incase they copied configs with a niceness preset, reset it to 0
        # so the backup doesn't fail before it starts.
        if UID != 0 and nice < 0:
            onetime.set('Options', 'Nice', 0)
        onetime.set('Options', 'Nice', int(self.ui.OneTimeNiceScale.get_value()))
        onetime.set('Options', 'Excludes', self.ui.OneTimeExcludesEntry.get_text())
            

    def _restore(self):
        onetime = config.OneTimeConf()
        t = onetime.get('Options', 'Recursive')
        if t == '1':
            self.ui.OneTimeRecursiveCheck.set_active(True)
        else:
            self.ui.OneTimeRecursiveCheck.set_active(False)
        t = onetime.get('Options', 'BackupHidden')
        if t == '1':
            self.ui.OneTimeBackupHiddenCheck.set_active(True)
        else:
            self.ui.OneTimeBackupHiddenCheck.set_active(False)
        t = onetime.get('Options', 'PkgListsToFile')
        if t == '1':
            self.ui.OneTimePkgListsToFileCheck.set_active(True)
        else:
            self.ui.OneTimePkgListsToFileCheck.set_active(False)
        t = onetime.get('Options', 'DiskInfoToFile')
        if t == '1':
            self.ui.OneTimeDiskInfoToFileCheck.set_active(True)
        else:
            self.ui.OneTimeDiskInfoToFileCheck.set_active(False)
        t = onetime.get('Options', 'Sparse')
        if t == '1':
            self.ui.OneTimeSparseCheck.set_active(True)
        else:
            self.ui.OneTimeSparseCheck.set_active(False)
        t = onetime.get('Options', 'FollowLinks')
        if t == '1':
            self.ui.OneTimeFollowLinksCheck.set_active(True)
        else:
            self.ui.OneTimeFollowLinksCheck.set_active(False)
        t = onetime.get('Options', 'Engine')
        if t == 'tar':
            self.ui.OneTimeArchiveEngineRadio.set_active(True)
        elif t == 'tar.gz':
            self.ui.OneTimeCompressedArchiveEngineRadio.set_active(True)
        elif t == 'rsync':
            self.ui.OneTimeDirectCopyEngineRadio.set_active(True)
        t = onetime.get('Options', 'Nice')
        self.ui.OneTimeNiceScale.set_value(float(t))
        self.ui.OneTimeExcludesEntry.set_text(onetime.get('Options', 'Excludes'))

    def startBackup(self):
        """ Backs up the paths """
        self.operationInProgress = True
        prefs = config.PrefsConf()
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.ui.trayNotify(_('Status'), _('Starting a one-time backup operation'), 5)
        self.ui.MainNotebook.set_current_page(2)
        self.ui.OneTimeRadioTool.set_active(True)
        onetime = config.OneTimeConf()
        onetime.set('Options', 'Destination', self.ui.OneTimeDestination.get_filename())
        self._save()
        #guiutils.busyCursor(self.ui.main)
        self.progressbar.set_text(_('Please wait...'))
        self.progressbar.startPulse()
        self._toggleLocked([self.ui.OneTimeRadioTool, self.ui.one_time_backup1])
        try:
            self.backupHandle = backend.OneTime(onetimeloc, self.logger)
            self.backupThread = guiutils.runFuncAsThread(self.backupHandle.backup)
            def updateProgress(self):
                """ Updates the statusbar """
                current, total = self.backupHandle.getProgress()
                if self.updateReturn == True and current != 0:
                    self.progressbar.set_text(_('Please wait... Backuping path %(a)i/%(b)i') % {'a': current, 'b': total})
                return self.updateReturn
            self.updateReturn = True
            updateTimeout = gobject.timeout_add(1000, updateProgress, self)
            while self.backupThread.isAlive() == True:
                while gtk.events_pending():
                    gtk.main_iteration()
                time.sleep(0.01)
        except Exception, e:
            errorDia = ErrorDia(\
                                self.ui.error_dia, \
                                self.ui.ErrorDiaLabel, \
                                _('An error occurred initializing the backup process.')\
                                )
            errorDia.run()
            errorDia.destroy()
            self.logger.logmsg('DEBUG', _('An error occurred initializing the backup process:\n%s') % str(e))
        self.updateReturn = False
        self.progressbar.stopPulse()
        self._toggleLocked([self.ui.OneTimeRadioTool, self.ui.one_time_backup1])
        for widget in [\
self.ui.new_set1, self.ui.backup_set_now1,\
self.ui.import_sets1, self.ui.export_sets1,\
self.ui.restore_set1, self.ui.restore_set2\
]:
            widget.set_sensitive(True)
        self.progressbar.set_text(_('Click \'Backup\' to begin'))
        #guiutils.normalCursor(self.ui.main)
        self.clear()
        if os.path.exists(onetimeloc):
            os.remove(onetimeloc)
        onetime = config.OneTimeConf(True)
        self.operationInProgress = False
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.ui.trayNotify(_('Status'), _('Finished the one-time backup operation'), 5)

    def cancelBackup(self):
        """ Cancels running onetime backup """
        self.updateReturn = False
        self.backupHandle.cancel()
        self.progressbar.set_text(_('Cancelling...'))



class ExportView(View):
    """ Wrapper for backup sets in a Treeview """
    def _on_toggled(self, cell, path, model):
        """ inverse toggle """
        #rendertoggle.set_active(not rendertoggle.get_active)
        model[path][0] = not model[path][0]

    def __init__(self, treeview, statusbar, ui):
        self.treeview = treeview
        self.logger = logging.getLogger(LOGGERNAME)
        self.statusbar= statusbar
        self.ui = ui
        self.liststore = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        # Give it columns
        cell = gtk.CellRendererToggle()
        cell.connect('toggled', self._on_toggled, self.liststore)
        cell.set_property('activatable', True)
        col = gtk.TreeViewColumn(_('Export'), cell, active=0)
        col.set_resizable(True)
        self.treeview.append_column(col)

        cell = gtk.CellRendererText()
        cell.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)
        col = gtk.TreeViewColumn(_('Set'), cell, text=1)
        col.set_resizable(True)
        self.treeview.append_column(col)

        self.treeview.set_model(self.liststore)
        self.treeview.set_reorderable(False)
        # Just to keep things clean.
        self._clear()

    def _clear(self):
        """Clear all sets from the view """
        self.liststore.clear()

    def _load(self):
        """ Load all set .conf files into the view """
        loaded_count = 0
        self.logger.logmsg('DEBUG', _('Parsing configuration files'))
        for root, dirs, files in os.walk(setloc):
            files.sort()
            for name in files:
                if name.endswith('.conf'):
                    self.liststore.append([True, name.split('.conf')[0]])
                else:
                    self.logger.logmsg('WARNING', \
_('Refusing to parse file \'%s\': configuration files must end in \'.conf\'') % name)

    def refresh(self):
        """ Clears & reloads the view contents """
        self._clear()
        self._load()            


class SetView(View):
    """ Wrapper for backup sets in an IconView """
    def __init__(self, iconview, statusbar, pathview, ui):
        # Declare...
        self.iconview = iconview
        self.logger = logging.getLogger(LOGGERNAME)
        self.statusbar = statusbar
        self.pathview = pathview
        self.ui = ui
        self.liststore = gtk.ListStore(gobject.TYPE_STRING, gtk.gdk.Pixbuf)
        self.cron = cron.CronTab()
        self.cron.environ = {'EDITOR':'%s/cronwriter.py' % MAIN_PATH}
        #self.gtkEventsThread = self.ui.gtkEventsThread
        self.progressbar = ProgressBar(self.ui.BackupSetProgress)
        # Setup Iconview
        self.iconview.set_model(self.liststore)
        self.iconview.set_reorderable(False)
        #self.iconview.set_columns(3)
        self.iconview.set_text_column(0)
        self.iconview.set_pixbuf_column(1)
        self.iconview.set_item_width(110)
        self.iconview.set_row_spacing(10)
        self.iconview.set_column_spacing(20)
        self.icon = self.ui.backupset_dia.render_icon(\
gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_DIALOG)
        # Keep things clean...
        self.refresh()

    def help(self):
        """ Display help  """
        try:
            import gnome
            gnome.help_display('fwbackups')
        except ImportError:
            webbrowser.open_new('http://www.diffingo.com/downloads/fwbackups/docs/html')
        except Exception, error:
            self.logger.logmsg('WARNING', _('Unexpected error \'%s\' while trying \
to show documentation!\nUsing online documentation instead.' % str(error)))
            webbrowser.open_new('http://www.diffingo.com/downloads/fwbackups/docs/html')

    # TODO: are these really needed? We set defaults in the .conf with
    # config.py why not just _restore() them?
    def __setTimesDefaults(self):
        self.ui.MonthOnlyBox.set_active(0)
        self.ui.DaysWeekOnlyBox.set_active(0)
        self.ui.HoursAtBox.set_active(0)
        self.ui.MinutesAtScale.set_value(0)
        self.ui.MonthFromBox1.set_active(0)
        self.ui.MonthFromBox2.set_active(0)
        self.ui.DaysWeekFromBox1.set_active(0)
        self.ui.DaysWeekFromBox2.set_active(0)
        self.ui.MonthAll.set_active(True)
        self.ui.DaysWeekAll.set_active(True)
        self.ui.HoursAt.set_active(True)
        self.ui.MinutesAt.set_active(True)
        self.ui.EasyConfigTable.set_sensitive(True)
        self.ui.EasyConfigExpander.set_expanded(True)
        self.ui.ManualConfigExpander.set_expanded(False)
        self.ui.ManualConfigTable.set_sensitive(False)
        self.ui.ManualMinuteEntry.set_text('')
        self.ui.ManualHourEntry.set_text('')
        self.ui.ManualMonthEntry.set_text('')
        self.ui.ManualDaysOfWeekEntry.set_text('')
        self.ui.ManualDaysOfMonthEntry.set_text('')

    def __setOptionsSimpleDefaults(self):
        self.ui.SetEnableCheck.set_active(True)
        self.ui.SetSparseCheck.set_active(False)
        self.ui.SetRecursiveCheck.set_active(True)
        self.ui.SetBackupHiddenCheck.set_active(True)
        self.ui.SetPkgListsToFileCheck.set_active(True)
        self.ui.SetDiskInfoToFileCheck.set_active(True)
        self.ui.SetFollowLinksCheck.set_active(False)
        self.ui.SetArchiveEngineRadio.set_active(True)
        self.ui.SetOldToKeepSpin.set_value(float(0))

    def __setOptionsAdvancedDefaults(self):
        self.ui.SetCommandBeforeEntry.set_text('')
        self.ui.SetCommandAfterEntry.set_text('')
        self.ui.SetExcludesEntry.set_text('')
        self.ui.SetNiceScale.set_value(float(0))

    def __setPathDefaults(self):
        self.ui.SetNameEntry.set_text('')
        self.ui.SetDestination.set_current_folder(userhome)
        self._checkDestPerms(userhome, self.ui.SetDestinationPermissionImage)

    def _setFieldDefaults(self):
        """ Reset all fields to the normals, as if never used """
        self.__setPathDefaults()
        self.__setTimesDefaults()
        self.__setOptionsSimpleDefaults()
        self.__setOptionsAdvancedDefaults()

    def _restore(self, set):
        """ Restore all the information from a .conf file """
        # Restore times
        t = set.get('Times', 'Custom')
        l = set.get('Times', 'Entry').split(' ')
        if t.strip() != 'False':
            self.ui.EasyConfigExpander.set_expanded(False)
            self.ui.ManualConfigExpander.set_expanded(True)
            self.ui.EasyConfigTable.set_sensitive(False)
            self.ui.ManualConfigTable.set_sensitive(True)
            try:
                self.ui.ManualMinuteEntry.set_text(l[0])
                self.ui.ManualHourEntry.set_text(l[1])
                self.ui.ManualDaysOfMonthEntry.set_text(l[2])
                self.ui.ManualMonthEntry.set_text(l[3])
                self.ui.ManualDaysOfWeekEntry.set_text(l[4])
            except IndexError:
                dialog = ErrorDia(self.ui.error_dia,\
self.ui.ErrorDiaLabel, _('Error parsing configuration file: \
The \'Custom\' option has errors.\nUsing default values instad.'))
                dialog.run()
                dialog.destroy()
                self.ui.SetManualMinuteEntry.set_text('')
                self.ui.SetManualHourEntry.set_text('')
                self.ui.SetManualDaysOfMonthEntry.set_text('')
                self.ui.SetManualMonthEntry.set_text('')
                self.ui.SetManualDaysOfWeekEntry.set_text('')
                self.__setEasyDefaults()
        else:
            self.ui.EasyConfigExpander.set_expanded(True)
            self.ui.ManualConfigExpander.set_expanded(False)
            self.ui.EasyConfigTable.set_sensitive(True)
            self.ui.ManualConfigTable.set_sensitive(False)
            # Minute            
            if l[0] == '*':
                self.ui.MinutesAll.set_active(True)
            else:
                self.ui.MinutesAt.set_active(True)
                self.ui.MinutesAtScale.set_value(float(l[0]))
            # Hour
            if l[1] == '*':
                self.ui.HoursAll.set_active(True)
            else:
                self.ui.HoursAt.set_active(True)
                self.ui.HoursAtBox.set_active(int(l[1]))
            # l[2] **Days of month would go here
            # Months
            t = l[3].split('-')
            if l[3] == '*':
                self.ui.MonthAll.set_active(True)
            elif len(t) == 1:
                self.ui.MonthOnly.set_active(True)
                self.ui.MonthOnlyBox.set_active(int(l[3]))
            elif len(t) == 3:
                self.ui.MonthFrom.set_active(True)
                self.ui.MonthFromBox1.set_active(int(t[1].split(',')[1]) - 1)
                self.ui.MonthFromBox2.set_active(int(t[1].split(',')[0]) - 1)
            else:
                self.ui.MonthFrom.set_active(True)
                self.ui.MonthFromBox1.set_active(int(t[0]) - 1)
                self.ui.MonthFromBox2.set_active(int(t[1]) - 1)
            # Day of Week
            t = l[4].split('-')
            if l[4] == '*':
                self.ui.DaysWeekAll.set_active(True)
            elif len(t) == 1:
                self.ui.DaysWeekOnly.set_active(True)
                self.ui.DaysWeekOnlyBox.set_active(int(l[4]))
            elif len(t) == 3:
                self.ui.DaysWeekFrom.set_active(True)
                self.ui.DaysWeekFromBox1.set_active(int(t[1].split(',')[1]))
                self.ui.DaysWeekFromBox2.set_active(int(t[1].split(',')[0]))
            else:
                
                self.ui.DaysWeekFrom.set_active(True)
                self.ui.DaysWeekFromBox1.set_active(int(t[0]))
                self.ui.DaysWeekFromBox2.set_active(int(t[1]))

        # Restore options
        t = set.get('Options', 'Destination')
        self.ui.SetDestination.set_current_folder(t)
        self._checkDestPerms(t, self.ui.SetDestinationPermissionImage)
        t = set.get('Options', 'Enabled')
        if t == '1':
            self.ui.SetEnableCheck.set_active(True)
        else:
            self.ui.SetEnableCheck.set_active(False)
        t = set.get('Options', 'Recursive')
        if t == '1':
            self.ui.SetRecursiveCheck.set_active(True)
        else:
            self.ui.SetRecursiveCheck.set_active(False)
        t = set.get('Options', 'PkgListsToFile')
        if t == '1':
            self.ui.SetPkgListsToFileCheck.set_active(True)
        else:
            self.ui.SetPkgListsToFileCheck.set_active(False)
        t = set.get('Options', 'DiskInfoToFile')
        if t == '1':
            self.ui.SetDiskInfoToFileCheck.set_active(True)
        else:
            self.ui.SetDiskInfoToFileCheck.set_active(False)
        t = set.get('Options', 'BackupHidden')
        if t == '1':
            self.ui.SetBackupHiddenCheck.set_active(True)
        else:
            self.ui.SetBackupHiddenCheck.set_active(False)
        t = set.get('Options', 'Sparse')
        if t == '1':
            self.ui.SetSparseCheck.set_active(True)
        else:
            self.ui.SetSparseCheck.set_active(False)
        t = set.get('Options', 'FollowLinks')
        if t == '1':
            self.ui.SetFollowLinksCheck.set_active(True)
        else:
            self.ui.SetFollowLinksCheck.set_active(False)
        t = set.get('Options', 'Sparse')
        t = set.get('Options', 'Engine')
        if t == "tar":
            self.ui.SetArchiveEngineRadio.set_active(True)
        elif t == "rsync":
            self.ui.SetDirectCopyEngineRadio.set_active(True)
        elif t == "tar.gz":
            self.ui.SetCompressedArchiveEngineRadio.set_active(True)
        self.ui.SetNiceScale.set_value(float(set.get('Options', 'Nice')))
        # Command entries
        self.ui.SetCommandBeforeEntry.set_text(set.get('Options', 'CommandBefore'))
        self.ui.SetCommandAfterEntry.set_text(set.get('Options', 'CommandAfter'))
        self.ui.SetOldToKeepSpin.set_value(float(set.get('Options', 'OldToKeep')))
        self.ui.SetExcludesEntry.set_text(set.get('Options', 'Excludes'))

    def regenerateCronTab(self):
        """ Regenerates a user's crontab """
        self.logger.logmsg('DEBUG', _('Regenerating crontab'))
        self.cron.clean()
        for root, dirs, files in os.walk(setloc):
            files.sort()
            for name in files:
                if name.endswith('.conf'):
                    set = config.BackupSetConf(name.split('.conf')[0])
                    #if set.get('Times', 'Custom') == 'True' or set.get('Times', 'Custom') == True:
                    #    l = set.get('Times', 'CustomEntry')
                    #else:
                    #    l = set.get('Times', 'Entry')
                    entry = set.get('Times', 'Entry')
                    self.cron.add(entry.split(' '))
                    self.logger.logmsg('DEBUG', _('Saving set \'%s\' to crontab') % name.split('.conf')[0])

    def _save(self, set):
        """ Save all the information to a .conf file, add to crontab """
        # Save paths
        set.remove_section('Paths')
        set.add_section('Paths')
        self.pathnumber = 0
        self.pathview.liststore.foreach(self._saveRow, set)
        origEntry = set.get('Times', 'Entry').split(' ')
        # Save times
        if self.ui.EasyConfigExpander.get_expanded():
            set.set('Times', 'Custom', 'False')
            entry = []
            # Order of appends must be same as crontab
            # Minutes
            if self.ui.MinutesAll.get_active():
                entry.append('*')
            if self.ui.MinutesAt.get_active():
                entry.append(str(int(self.ui.MinutesAtScale.get_value())))
            # Hours
            if self.ui.HoursAll.get_active():
                entry.append('*')
            if self.ui.HoursAt.get_active():
                entry.append(str(self.ui.HoursAtBox.get_active()))
            # Days of month
            entry.append('*')
            # Months
            if self.ui.MonthAll.get_active():
                entry.append('*')
            elif self.ui.MonthOnly.get_active():
                entry.append(str(self.ui.MonthOnlyBox.get_active() + 1))
            elif self.ui.MonthFrom.get_active():
                from1 = int(self.ui.MonthFromBox1.get_active()) + 1
                from2 = int(self.ui.MonthFromBox2.get_active()) + 1
                if from1 > from2:
                    entry.append('1-%i,%i-12' % (from2, from1))
                else:
                    entry.append('%i-%i' % (from1, from2))
            # Days of week
            if self.ui.DaysWeekAll.get_active():
                entry.append('*')
            if self.ui.DaysWeekOnly.get_active():
                entry.append(str(self.ui.DaysWeekOnlyBox.get_active()))
            if self.ui.DaysWeekFrom.get_active():
                from1 = int(self.ui.DaysWeekFromBox1.get_active())
                from2 = int(self.ui.DaysWeekFromBox2.get_active())
                if from1 > from2:
                    entry.append('0-%i,%i-7' % (from2, from1))
                else:
                    entry.append('%i-%i' % (from1, from2))
        else:
            set.set('Times', 'Custom', 'True')
            entry = []
            entry.append(self.ui.ManualMinuteEntry.get_text().strip())
            entry.append(self.ui.ManualHourEntry.get_text())
            entry.append(self.ui.ManualDaysOfMonthEntry.get_text().strip())
            entry.append(self.ui.ManualMonthEntry.get_text().strip())
            entry.append(self.ui.ManualDaysOfWeekEntry.get_text().strip())
        entry.append('fwbackups-run \'%s\'' % set.get_conffile().split('/')[-1].split('.conf')[0])
        try:
            if self.cron.search(origEntry):
                self.cron.edit(origEntry, entry)
            else:
                self.cron.add(entry)
        except Exception, e:
            errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel, \
_('Error writing to the crontab because: %s') % str(e) )
            errorDialog.run()
            errorDialog.destroy()
            return
        set.set('Times', 'Entry', ' '.join(entry))
        # Save options
        set.set('Options', 'Destination', self.ui.SetDestination.get_filenames()[0])
        if self.ui.SetEnableCheck.get_active():
            set.set('Options', 'Enabled', 1)
        else:
            set.set('Options', 'Enabled', 0)
        if self.ui.SetRecursiveCheck.get_active():
            set.set('Options', 'Recursive', 1)
        else:
            set.set('Options', 'Recursive', 0)
        if self.ui.SetPkgListsToFileCheck.get_active():
            set.set('Options', 'PkgListsToFile', 1)
        else:
            set.set('Options', 'PkgListsToFile', 0)
        if self.ui.SetDiskInfoToFileCheck.get_active():
            set.set('Options', 'DiskInfoToFile', 1)
        else:
            set.set('Options', 'DiskInfoToFile', 0)
        if self.ui.SetBackupHiddenCheck.get_active():
            set.set('Options', 'BackupHidden', 1)
        else:
            set.set('Options', 'BackupHidden', 0)
        if self.ui.SetSparseCheck.get_active():
            set.set('Options', 'Sparse', 1)
        else:
            set.set('Options', 'Sparse', 0)
        if self.ui.SetFollowLinksCheck.get_active():
            set.set('Options', 'FollowLinks', 1)
        else:
            set.set('Options', 'FollowLinks', 0)
        if self.ui.SetArchiveEngineRadio.get_active():
            engine = 'tar'                
        elif self.ui.SetDirectCopyEngineRadio.get_active():
            engine = 'rsync'                
        elif self.ui.SetCompressedArchiveEngineRadio.get_active():
            engine = 'tar.gz'
        set.set('Options', 'Engine', engine)
        set.set('Options', 'CommandBefore', self.ui.SetCommandBeforeEntry.get_text())
        set.set('Options', 'CommandAfter', self.ui.SetCommandAfterEntry.get_text())
        set.set('Options', 'OldToKeep', self.ui.SetOldToKeepSpin.get_value())
        set.set('Options', 'Excludes', '%s' % self.ui.SetExcludesEntry.get_text())
        nice = int(self.ui.SetNiceScale.get_value())
        set.set('Options', 'Nice', nice)
        # Incase they copied configs with a niceness preset, reset it to 0
        # so the backup doesn't fail before it starts.
        if UID != 0 and nice < 0:
            set.set('Options', 'Nice', 0)

    def _saveRow(self, model, path, iter, config, user_data=False):
        """ Saves one row to the configuration """
        pathno = 'path' + str(self.pathnumber)
        if not user_data:
            user_data = model.get_value(iter, 1)
        config.set('Paths', pathno, user_data)
        self.pathnumber = int(self.pathnumber + 1)

    def _checkExistingOrNone(self, name, parent, entry):
        """ Check if a filename exists, or if none is set. """
        while os.path.exists('%s/%s.conf' % (setloc, name)):
            dialog = ConfirmDia(\
                                self.ui.confirm_dia, \
                                self.ui.ConfirmDiaLabel, \
                                _('The set \'%s\' already exists. Overwrite?') \
                                % (name)\
                                )
            response = dialog.run()
            dialog.destroy()
            if response == gtk.RESPONSE_YES:
                return self._checkExistingOrNone(name, parent, entry)
            elif response == gtk.RESPONSE_NO:
                parentresponse = parent.run()
                name = entry.get_text()
                if parentresponse == gtk.RESPONSE_CANCEL:
                    return False
                
            while name == None or name == "":
                dialog = ErrorDia(\
                                  self.ui.error_dia, \
                                  self.ui.ErrorDiaLabel, \
                                  _('Please enter a set name.')\
                                  )
                response = dialog.run()
                dialog.destroy()
            if response == gtk.RESPONSE_YES:
                return self._checkExistingOrNone(name, parent, entry)
            elif response == gtk.RESPONSE_NO:
                parentresponse = parent.run()
                name = entry.get_text()
                if parentresponse == gtk.RESPONSE_CANCEL:
                    return False
                
        return name

    def new(self):
        """ New set. """
        self.pathview.clear()
        self._setFieldDefaults()
        backupDialog = BackupSetDia(self.ui.backupset_dia)
        response = backupDialog.run()
        if response == gtk.RESPONSE_HELP:
            self.help()
            response = backupDialog.run()
        if response == gtk.RESPONSE_OK:
            name = self._checkExistingOrNone(self.ui.SetNameEntry.get_text(),\
self.ui.backupset_dia, self.ui.SetNameEntry)
            set = config.BackupSetConf(name, True)
            self._save(set)
            self.refresh()
            self.statusbar.newmessage(_('Creating set \'%s\'') % (name), 3)
            self.logger.logmsg('DEBUG', _('Creating set \'%s\'') % (name))
        backupDialog.destroy()
        self.refresh()

    def edit(self):
        """ Edit a set. """
        self.pathview.clear()
        self._setFieldDefaults()
        try:
            # Get iterator from iconvew to get name
            selected = guiutils.getSelectedItems(self.iconview)[0]
            iterator = self.liststore.get_iter(selected)
            name = self.liststore.get_value(iterator, 0)
            self.ui.SetNameEntry.set_text(name)
            # Now load the set & refresh pathviews...
            set = config.BackupSetConf(name)
            self.pathview.refresh(set)
        except IndexError:
            self.statusbar.newmessage(_('Please select a set before choosing an action.'), 3)
            return
        self._restore(set)
        backupDialog = BackupSetDia(self.ui.backupset_dia)
        response = backupDialog.run()
        if response == gtk.RESPONSE_HELP:
            self.help()
            response = backupDialog.run()
        if response == gtk.RESPONSE_OK:
            newname = self.ui.SetNameEntry.get_text()
            if newname != name:
                newname = self._checkExistingOrNone(newname, self.ui.backupset_dia, self.ui.SetNameEntry)
                namepath = utils.ConvertPath('%s/%s.conf' % (setloc, name))
                newnamepath = utils.ConvertPath('%s/%s.conf' % (setloc, newname))
                try:
                    os.rename(namepath, newnamepath)
                    set.set_conffile(newnamepath)
                    oldcron = set.get('Times', 'Entry').split(' ')
                    newcron = set.get('Times', 'Entry').split(' ')
                    for i in newcron[6:]:
                        newcron.remove(i)
                    newcron.append('\'%s\'' % newname)
                    try:
                        if not self.cron.search(oldcron):
                            self.cron.add(newcron)
                        self.cron.edit(oldcron, newcron)
                    except Exception, e:
                        errorDialog = ErrorDia(self.ui.error_dia, \
self.ui.ErrorDiaLabel, _('Error writing to the crontab because:') % (str(e)))
                        errorDialog.run()
                        errorDialog.destroy()
                        return
                    set.set('Times', 'Entry', ' '.join(newcron))
                    self.logger.logmsg('DEBUG', _('Renaming set \'%(a)s\' to \'%(b)s\'') % {'a': name, 'b': newname})
                except:
                    raise
                    errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel,\
_('Cannot rename set \'%(a)s\' to \'%(b)s\' because an error occurred.') % {'a': name, 'b': newname})
                    errorDialog.run()
                    errorDialog.destroy()
                    try:
                        os.rename(newnamepath, namepath)
                    except:
                        pass
            else:
                newname = name
            self._save(set)
            self.logger.logmsg('DEBUG', _('Saving changes to set \'%s\'') % (newname))
            self.statusbar.newmessage(_('Saving changes to set \'%s\'') % (newname), 3)
        backupDialog.destroy()
        self.refresh()

    def remove(self):
        """ Removes a set. """
        try:
            selected = guiutils.getSelectedItems(self.iconview)[0]
            iterator = self.liststore.get_iter(selected)
            name = self.liststore.get_value(iterator, 0)
        except IndexError:
            self.statusbar.newmessage(_('Please select a set before choosing an action.'), 3)
            return
        confirmDialog = ConfirmDia(self.ui.confirm_dia, self.ui.ConfirmDiaLabel, \
_('You are about to permenantly remove a backup set.\nAre you sure you would like to continue?'))
        response = confirmDialog.run()
        if response == gtk.RESPONSE_YES:
            try:
                set = config.BackupSetConf(name)
                l = set.get('Times', 'Entry').split(' ')
                os.remove(utils.ConvertPath('%s/%s.conf' % (setloc, name)))
                self.cron.remove(l)
                self.statusbar.newmessage(_('Removing set \'%s\'') % (name), 3)
                self.logger.logmsg('DEBUG', _('Removing set \'%s\'') % name)
                self.refresh()
            except OSError, e:
                errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel,\
_('Error removing set \'%(a)s\' because %(b)s') % {'a': name, 'b': str(e)})
                errorDialog.run()
                errorDialog.destroy()
                self.refresh()
            except Exception, e:
                errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel,\
_('Error removing set \'%(a)s\' because %(b)s') % {'a': name, 'b': str(e)})
                errorDialog.run()
                errorDialog.destroy()
        confirmDialog.destroy()

    def _load(self):
        """ Load all set .conf files """
        loaded_count = 0
        self.logger.logmsg('DEBUG', _('Parsing configuration files'))
        for root, dirs, files in os.walk(setloc):
            files.sort()
            for name in files:
                if name.endswith('.conf'):
                    self.liststore.append([name.split('.conf')[0], self.icon])
                    self.ui.RestoreSetNameBox.get_model().append([name.split('.conf')[0]])
                    self.ui.RestoreSetNameBox.set_active(0)
                    self.ui.RestoreSetDateBox.set_active(0)
                    loaded_count += 1
                else:
                    self.logger.logmsg('WARNING', _('Refusing to parse file \
\'%s\': configuration files must end in \'.conf\'') % name)
        self.ui.TotalSetsLabel.set_text(str(loaded_count))
        self.ui.TotalSetsLabel.set_use_markup(True)

    def _clear(self):
        """Clear all sets from the view """
        self.liststore.clear()
        self.ui.RestoreSetNameBox.get_model().clear()

    def refresh(self):
        """ Clear + reload the treeview """
        self._clear()
        self._load()

    def get_enabled(self):
        """ Is the selected .conf enabled? """
        selected = guiutils.getSelectedItems(self.iconview)
        if selected == False or len(selected) <= 0:
            return "none"
        else:
            iterator = self.liststore.get_iter(selected[0])
            name = self.liststore.get_value(iterator, 0)
        set = config.BackupSetConf(name)
        enabled = set.get('Options', 'Enabled')
        if str(enabled) == '1':
            return True
        else:
            return False

    def set_enabled(self, bool):
        """ Set the selected .conf's enabled tag """
        try:
            selected = guiutils.getSelectedItems(self.iconview)[0]
            iterator = self.liststore.get_iter(selected)
            name = self.liststore.get_value(iterator, 0)
            set = config.BackupSetConf(name)
            if bool:
                set.set('Options', 'Enabled', 1)
            else:
                set.set('Options', 'Enabled', 0)
        except:
            pass

    def _toggleLocked(self, ignore):
        """ Toggle locking in the UI """
        for widget in   [
self.ui.BackupSetsRadioTool,
self.ui.OneTimeRadioTool, self.ui.RestoreRadioTool,
self.ui.vbuttonbox1, self.ui.OneTimeVButtonBox,
self.ui.BackupSetCancelButton, self.ui.OneTimeDestination,
self.ui.BackupSetIconview,
self.ui.OneTimeOptionsButton,
self.ui.new_set1, self.ui.backup_set_now1,
self.ui.import_sets1, self.ui.backup_sets1,
self.ui.export_sets1, self.ui.one_time_backup1,
self.ui.restore1, self.ui.restore_set1,
self.ui.restore_set2
]:
            try:
                ignore.index(widget)
            except ValueError:
                widget.set_sensitive(not widget.get_property('sensitive'))
        for widget in [
self.ui.new_set1, self.ui.backup_set_now1,
self.ui.import_sets1,
self.ui.export_sets1, self.ui.restore_set1,
self.ui.restore_set2, self.ui.edit_set1,
self.ui.remove_set1, self.ui.duplicate_set1
]:
            widget.set_sensitive(False)

    def startBackup(self):
        self.operationInProgress = True
        self.ui.MainNotebook.set_current_page(1)
        self.ui.BackupSetsRadioTool.set_active(True)
        try:
            selected = self.iconview.get_selected_items()[0]
            iterator = self.liststore.get_iter(selected)
            name = self.liststore.get_value(iterator, 0)
        except IndexError:
            self.statusbar.newmessage(_('Please select a set before choosing an action.'), 3)
            return
        prefs = config.PrefsConf()
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.ui.trayNotify(_('Status'), _('Starting a backup operation of set \'%(a)s\'' % {'a': name}), 5)
        self._toggleLocked([self.ui.BackupSetsRadioTool, self.ui.backup_sets1])
        #guiutils.busyCursor(self.ui.main)
        self.progressbar.startPulse()
        self.progressbar.set_text(_('Please wait...'))
        try:
            self.backupHandle = backend.Automated(name, self.logger)
            self.backupThread = guiutils.runFuncAsThread(self.backupHandle.backup)
            def updateProgress(self):
                """ Updates the statusbar """
                current, total = self.backupHandle.getProgress()
                if self.updateReturn == True and current != 0:
                    self.progressbar.set_text(_('Please wait... Backuping path %(a)i/%(b)i') % {'a': current, 'b': total})
                return self.updateReturn
            self.updateReturn = True
            updateTimeout = gobject.timeout_add(1000, updateProgress, self)
            while self.backupThread.isAlive() == True:
                while gtk.events_pending():
                    gtk.main_iteration()
                time.sleep(0.01)
        except Exception, e:
            errorDia = ErrorDia(\
                                self.ui.error_dia, \
                                self.ui.ErrorDiaLabel, \
                                _('An error occurred initializing the backup process.')\
                                )
            errorDia.run()
            errorDia.destroy()
            self.logger.logmsg('DEBUG', _('An error occurred initializing the backup process:\n%s') % str(e))
        self.updateReturn = False
        self.progressbar.stopPulse()
        #guiutils.normalCursor(self.ui.main)
        self._toggleLocked([self.ui.BackupSetsRadioTool, self.ui.backup_sets1])
        for widget in [\
self.ui.new_set1, self.ui.backup_set_now1,\
self.ui.import_sets1, self.ui.export_sets1,\
self.ui.restore_set1, self.ui.restore_set2\
]:
            widget.set_sensitive(True)
        self.progressbar.set_text(_('Click \'Backup Set Now\' to begin'))
        self.refresh()
        self.operationInProgress = False
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.ui.trayNotify(_('Status'), _('Finished the backup operation of set \'%(a)s\'' % {'a': name}), 5)

    def cancelBackup(self):
        self.updateReturn = False
        self.backupHandle.cancel()
        self.progressbar.set_text(_('Cancelling...'))


