#! /usr/bin/python
# -*- coding: utf-8 -*-
#    Copyright (C) 2005, 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Puts it all together.
"""
import gtk
import gtk.glade
import gobject
import getopt
import os
import sys
import logging
try:
    import pynotify
    NOTIFY_AVAIL = True
    pynotify.init('fwbackups')
except:
    NOTIFY_AVAIL = False

import re

from fwbackups.i18n import _
from fwbackups.const import *
from fwbackups import fwlogger, config
from fwbackups import guicallbacks, guiwidgets, shutil_modded, fwbackupsError
gobject.threads_init()

class fwbackups(guicallbacks.Callbacks):
    """ Main program """
    def __init__(self, verbose):
        guicallbacks.Callbacks.__init__(self)
        self.operationInProgress = True
        try:
            import gnome
            prog = gnome.program_init('fwbackups', VERSION, properties={gnome.PARAM_APP_DATADIR : '/usr/share'})
        except ImportError:
            pass
        config._setupConf()
        self.verbose = verbose
        if verbose == True:
            level = logging.DEBUG
        else:
            level = logging.INFO
        try:
            self.logger, self.loghandler = self._setupLogger(level)
        except fwbackupsError, e:
            print _('Error setting up the log. Aborting.')
            print _('Reason: %s' % str(e))
            sys.exit(1)
        # Don't show tabs on main...
        self.ui.MainNotebook.set_show_tabs(False)
        # render icons: about dialog
        self.ui.image1.set_from_file('/usr/share/pixmaps/fwbackups.svg')
        self.ui.main.set_icon_from_file('/usr/share/pixmaps/fwbackups.svg')
        #self._setupTrayIcon()
        self.statusbar = guiwidgets.StatusBar(self.ui.statusbar1)
        # Creating the various views
        self.OneTimeView1 = guiwidgets.OneTimeView(self.ui.OneTimeTreeview, self.statusbar, self.ui)
        self.SetEditorPathView1 = guiwidgets.PathView(self.ui.BackupDiaTreeview, self.statusbar, self.ui)
        self.SetView1 = guiwidgets.SetView(self.ui.BackupSetIconview, self.statusbar, self.SetEditorPathView1, self.ui)
        self.ExportView1 = guiwidgets.ExportView(self.ui.ExportTreeview, self.statusbar, self.ui)
        # Defaults, labels...
        self._setDefaults()
        self._setLabels()
        # Parent windows... FIXME: What if we call a confirm on non-main window?
        self.ui.about.set_transient_for(self.ui.main)
        self.ui.path_dia.set_transient_for(self.ui.main)
        self.ui.backupset_dia.set_transient_for(self.ui.main)
        self.ui.confirm_dia.set_transient_for(self.ui.main)
        self.ui.error_dia.set_transient_for(self.ui.main)
        self.ui.path_dia.set_transient_for(self.ui.main)
        self.ui.save_dia.set_transient_for(self.ui.main)
        self.ui.export_dia.set_transient_for(self.ui.main)
        self.ui.options_dia.set_transient_for(self.ui.main)
        self.ui.preferences_dia.set_transient_for(self.ui.main)
        prefs = config.PrefsConf(create=True, logger=self.logger)
        self._setupTrayIcon()
        if int(prefs.get('Preferences', 'ShowTrayIcon')) == 1:
            self.trayicon.set_visible(True)
        else:
            self.trayicon.set_visible(False)
        # Welcome text = username
        self.ui.WelcomeLabel.set_text(_('Welcome, %s'  % USER))
        # We're not running a backup
        self.operationInProgress = False
        # Clean up, clean up, everybody do your share...
        fh = open('/etc/crontab', 'r')
        crontab = fh.read()
        if re.search('.*fwbackups.*', crontab):
            if int(prefs.get('Preferences', 'DontShowMe_OldVerWarn')) == 0:
                infoDialog = guiwidgets.InfoDia(self.ui.info_dia, self.ui.InfoDiaLabel, _('fwbackups has detected you were previously using and older version of fwbackups. If you are the system administrator, please make sure to run fwbackups as root and use the \n<i>File > Import</i>\nfunction to import your old configuration to version 1.43.0 as well as clean up the  system crontab file. If this action is not completed, the backup from version 1.42.x <b>will not run automatically</b>.\n<i>Tip: The v1.42.x configuration is stored in /etc/fwbackups</i>'), dontShowMe=self.ui.InfoDiaDontAgainCheck)
                infoDialog.run()
                if self.ui.InfoDiaDontAgainCheck.get_active():
                    prefs.set('Preferences', 'DontShowMe_OldVerWarn', 1)
                infoDialog.destroy()
        if UID == 0:
            try:
                shutil_modded.copy('/etc/crontab', '/etc/crontab.fwbk')
                if re.search('.*fwbackups.*', crontab):
                    self.logger.logmsg('INFO', _('Cleaning up /etc/crontab from previous versions of fwbackups'))
                    p = re.compile('.*fwbackups.*')
                    newcrontab = p.sub('', crontab)
                    fh.close()
                    fh = open('/etc/crontab', 'w')
                    fh.write(newcrontab)
            except Exception, e:
                errorDialog = guiwidgets.ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel, _('An error occurred while trying to cleanup the crontab:\n%s\nThe system crontab (/etc/crontab) may be left in an inconsitent state. A backup copy of the original crontab has been created in /etc/crontab.fwbak') % str(e))
                errorDialog.run()
                errorDialog.destroy()
                self.logger.logmsg('ERROR', _('An error occurred while trying to cleanup the crontab:\n%s\nThe system crontab (/etc/crontab) may be left in an inconsitent state. A backup copy of the original crontab has been created as /etc/crontab.fwbak') % str(e))
        else:
            self.ui.SetNiceScale.get_adjustment().set_all(0, 0, 19, 1, 5, 0)
            self.ui.OneTimeNiceScale.get_adjustment().set_all(0, 0, 19, 1, 5, 0)
        # Log size...
        self._checkLogSize()
        
        # Welcome...
        self.logger.logmsg('INFO', _('fwbackups administrator started'))

    def _checkLogSize(self):
        """ Check the log size, clear if needed """
        statsize = int(os.stat(logloc)[6])
        if statsize >= 1048577:
            #1048576 is a MB. Log will be literally bigger!
            size = '%s KB' % (statsize / 1024)
            confirmDialog = guiwidgets.ConfirmDia(self.ui.confirm_dia, self.ui.ConfirmDiaLabel, _('The log file is becoming large (%s). Would you like to clear it?') % size)
            response = confirmDialog.run()
            if response == gtk.RESPONSE_YES:
                self.loghandler.clear()
                self.logger.get_label().set_text(str(0))
                self.logger.logmsg('INFO', _('Log cleared'))
            confirmDialog.destroy()

    def _setLabels(self):
        """ Set label around the program to the correct values """
        #label = self.ui.AboutDiffingoComButton.get_label()
        #label.set_text('<span color="blue">www.diffingo.com</span>')
        #label.set_use_markup(True)
        self.ui.AboutVersionLabel.set_text('<span size="xx-large" weight="bold">fwbackups %s</span>' % VERSION)
        self.ui.AboutVersionLabel.set_use_markup(True)

    def _setupTrayIcon(self):
        """ Sets up the tray icon """
        #pix = self.ui.main.render_icon(gtk.STOCK_COPY, gtk.ICON_SIZE_MENU)
        pix = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/fwbackups.svg")
        self.trayicon = gtk.status_icon_new_from_pixbuf(pix)
        self.trayicon.set_from_pixbuf(pix)
        #self.trayicon = gtk.status_icon_new_from_stock(gtk.STOCK_COPY)
        self.trayicon.connect("popup_menu", self._Popup)
        self.trayicon.connect("activate", self._clicked)
        self.trayicon.set_tooltip('fwbackups')

    def getTrayIcon(self):
        """ Returns the tray icon object """
        return self.trayicon

    def _clicked(self, status):
        """ Tray icon is clicked.
            status: the gtk.StatusIcon
        """
        """def menu_pos(menu):
            return gtk.status_icon_position_menu(menu, self.trayicon)
        self.ui.TrayMenu1.popup(None, None, menu_pos, 0,
                                gtk.get_current_event_time())"""
        if self.ui.main.get_property('visible'):
          self.ui.main.hide()
          self.ui.show_administrator1.set_active(False)
        else:
          self.ui.main.show()
          self.ui.show_administrator1.set_active(True)
        #self.ui.TrayMenu1.show()

    def _Popup(self, status, button, time):
        """ Popup the menu at the right position """
        def menu_pos(menu):
            return gtk.status_icon_position_menu(menu, self.trayicon)
        
        self.ui.TrayMenu1.popup(None, None, menu_pos, button, time)
        self.ui.TrayMenu1.show()

    def trayNotify(self, summary, body, timeout=0):
        """ Display a notification attached to the tray icon if applicable """
        if NOTIFY_AVAIL == False:
            return self.logger.logmsg('WARNING', _('pynotify could not be imported; Notifcations will not be displayed. Please make sure you have libnotify notify-python or pynotify installed.'))
        # text
        notify = pynotify.Notification(summary, body)
        # icon
        pix = self.ui.main.render_icon(gtk.STOCK_COPY, gtk.ICON_SIZE_DIALOG)
        notify.set_icon_from_pixbuf(pix)
        # location
        tray = self.getTrayIcon()
        if tray.get_visible():
            notify.set_property('status-icon', tray)
        #else:
        #    notify.attach_to_widget(tray)
        # timeout?
        if timeout != 0:
            notify.set_timeout(int(timeout) * 1000)
        # finally, show it
        notify.show()

    def _setupLogger(self, level):
        """ Setup the fwbackups logger """
        logging.setLoggerClass(fwlogger.Logger)
        # Create logger and set level to debug
        log = logging.getLogger(LOGGERNAME)
        log.setLevel(level)
        # Create console & handler from console
        logconsole = guiwidgets.TextViewConsole(self.ui.LogViewerTextview)
        loghandler = guiwidgets.TextViewLogHandler(logconsole)
        # Create formatter & add formatter to handler
        logformatter = logging.Formatter("%(message)s")
        loghandler.setFormatter(logformatter)
        # add handler to logger
        log.addHandler(loghandler)
        log.connect(self.updateErrorWarnLabel)
        buffer = logconsole.buffer
        start = buffer.get_start_iter()
        end = buffer.get_end_iter()
        self.ui.ErrorsInLogLabel.set_text('0')
        self.ui.FailedBackupRunsLabel.set_text('0')
        for i in buffer.get_text(start, end).split('\n'):
            errType = i.split(' ')
            if len(errType) >= 5:
                if errType[4] == _('ERROR'):
                    self.updateErrorWarnLabel('error',  i, False)
                elif errType[4] == _('WARNING'):
                    self.updateErrorWarnLabel('warning', i, False)
                #if errType[4] == _('WARNING') and i.endswith(_('Backup failed!')):
                    #self.updateFailedBackupRunsLabel(self)
        return log, loghandler
            
    def _setDefaults(self):
        if USER != 'root':
            self.ui.SetDiskInfoToFileCheck.set_active(False)
            self.ui.SetDiskInfoToFileCheck.set_sensitive(False)
            self.ui.OneTimeDiskInfoToFileCheck.set_active(False)
            self.ui.OneTimeDiskInfoToFileCheck.set_sensitive(False)
        # Default to page 0...
        self.ui.AboutNotebook.set_current_page(0)
        self.ui.MainNotebook.set_current_page(0)
        self.ui.BackupDiaNotebook.set_current_page(0)
        # Default fields...
        self.ui.RestoreSetNameBox.set_active(0)
        self.ui.RestoreSetDateBox.set_active(0)
        # Default Labels...
        self._setLabels()
        # Default progressbars...
        self.SetView1.progressbar.set_text(_('Click \'Backup Set Now\' to begin'))
        self.OneTimeView1.progressbar.set_text(_('Click \'Backup\' to begin'))
        self.ui.RestoreProgress.set_text(_('Click \'Restore\' to begin'))
        # Restore Dest to reset "Not Loaded" perm check
        self.ui.RestoreDestination.set_current_folder(userhome)

def usage():
    """ Print the application usage """
    print _('Usage: fwbackups [OPTIONS]\n\
  Valid Options:\n\
    -h, --help  :  Print this message and exit\n\
    -v, --verbose  :  Increase verbosity (print debug messages)\n\
')

# Only if we're in main execution
if __name__ == "__main__":
    verbose = False
    try:
        avalableOptions = ["help", "verbose"]
        # letter = plain options
        # letter: = option with arg
        (opts, rest_args) = getopt.getopt(sys.argv[1:],"hv", avalableOptions)
    except (getopt.GetoptError), e:
        print e, ""
        usage()
        sys.exit(1)
    # Parse args, take action
    if opts == []:
        pass
    else:
        for (opt, value) in opts:
            if opt == "-h" or opt == "--help":
                usage()
                sys.exit(1)
            elif opt == "-v" or opt == "--verbose":
                verbose = True
            """if opt == "-X" or opt == "--X":
                dostuffhere(value)"""

    try:
        # Startup the application and call the gtk event loop
        MainApp = fwbackups(verbose)
        gtk.main()
    except KeyboardInterrupt:
        # ctrl+c?
        MainApp.main_close(None)
